# -*- coding: utf8 -*-

from django.conf import settings
from django.http import HttpResponse, HttpResponseForbidden, Http404
from django.shortcuts import render, get_object_or_404, redirect
from django.core.cache import cache

from lib.access_auth import permission_needed
from lib.tools import confirmation
from .models import Evenement, liste_evts_avec_videos
from portailuser.models import Ecole
from live.forms import EvenementForm

from datetime import datetime, timedelta
from socket import gethostbyaddr
import os
import time
import hashlib
import json


MDP_SEL = "da39a3ee5e6b4b0d3255bfef95601890afd80709"
STREAMS = (
    {
        "ident": "240p",
        "defaut": False,
        "qualite": "240p",
        "url": "https://inpflix.inpt.fr/240p/{name}.m3u8",
        "largeur": 854,
        "hauteur": 480,
    },
    {
        "ident": "360p",
        "defaut": False,
        "qualite": "360p",
        "url": "https://inpflix.inpt.fr/360p/{name}.m3u8",
        "largeur": 854,
        "hauteur": 480,
    },
    {
        "ident": "480p",
        "defaut": False,
        "qualite": "480p",
        "url": "https://inpflix.inpt.fr/480p/{name}.m3u8",
        "largeur": 854,
        "hauteur": 480,
    },
    {
        "ident": "720p",
        "defaut": True,
        "qualite": "720p",
        "url": "https://inpflix.inpt.fr/720p/{name}.m3u8",
        "largeur": 854,
        "hauteur": 480,
    },
    {
        "ident": "1080p",
        "defaut": False,
        "qualite": "1080p",
        "url": "https://inpflix.inpt.fr/1080p/{name}.m3u8",
        "largeur": 854,
        "hauteur": 480,
    },
)


def mdp_stream():
    m = hashlib.sha256()
    m.update((MDP_SEL + datetime.now().strftime("%Y%m%d")).encode("utf8"))
    return m.hexdigest()[:8]


def page_live(request, live_prive):
    request.desactiver_widgets = True

    stream_utilise = None
    qualites = []

    for stream in STREAMS:
        qualites.append((stream["ident"], stream["qualite"]))
        if stream["ident"] in request.GET or (stream_utilise is None and stream["defaut"]):
            stream_utilise = stream

    if live_prive and not request.user.is_authenticated:
        return render(request, "live/live_auth.html")

    if live_prive:
        url = stream_utilise["url"].format(name=mdp_stream())
    elif "cle" in request.GET:
        url = stream_utilise["url"].format(name=request.GET["cle"])
    else:
        url = stream_utilise["url"].format(name="live")

    c = {
        "stream": stream_utilise,
        "qualites": qualites,
        "url": url,
        "url_dirname": os.path.dirname(url),
        "url_basename": os.path.basename(url),
        "taille": (stream_utilise["largeur"], stream_utilise["hauteur"]),
    }

    return render(request, "live/live.html", c)


def accueil(request):
    request.desactiver_widgets = True
    live = Evenement.objects.filter(est_cloture=False, date_effective__lte=datetime.today())[0:1]

    if live:
        live = live[0]
        return page_live(request, live.prive)

    prochain_live = Evenement.objects.filter(est_cloture=False).order_by("date_effective")[0:1]
    if prochain_live:
        prochain_live = prochain_live[0]
    else:
        prochain_live = None

    evenements = Evenement.objects.order_by("-date_debut")[0:10]
    liste = liste_evts_avec_videos()
    for evenement in evenements:
        if evenement.nom_interne in liste:
            evenement.vid_dispo = True

    c = {
        "evenements": evenements,
        "prochain_live": prochain_live,
    }

    reponse = render(request, "live/accueil.html", c)

    if prochain_live:
        delai = abs(prochain_live.delta_retard())
        reponse["Refresh"] = "120" if delai > 120 else str(delai)

    return reponse


def video_evenement(request, ident):
    request.desactiver_widgets = True
    evenement = get_object_or_404(Evenement, nom_interne=ident)

    if not request.user.is_authenticated and evenement.prive:
        response = render(request, "auth/access_restricted.html")
        response.status_code = 401
        return response

    if evenement.nom_interne not in liste_evts_avec_videos():
        raise Http404

    return render(request, "live/video_evenement.html", locals())


def etat_live(request):
    """
    Retourne 'fin' quand le live est terminé, 'ok' si il est en cours et que le
    stream fonctionne, et 'ko' quand il est en cours et que le stream est arreté.
    Permet de recharger la page du live quand le stream reprend après une coupure,
    et de signaler aux utilisateurs quand le live est terminé.
    """

    stream = request.GET.get("stream", "")

    live = Evenement.objects.filter(est_cloture=False, date_effective__lte=datetime.today())[0:1]
    if live:
        if cache.get(f"live_stream_{stream}_actif", False):
            return HttpResponse("ok", content_type="text/plain")

        return HttpResponse("ko", content_type="text/plain")

    return HttpResponse("fin", content_type="text/plain")


@permission_needed("gerer_live")
def liste(request):
    if request.ecole != Ecole.n7:
        raise Http404

    c = {
        "evenements": Evenement.objects.all()[0:10],
    }
    return render(request, "live/liste.html", c)


@permission_needed("gerer_live")
def edition(request, ident=None):
    if request.ecole != Ecole.n7:
        raise Http404

    if ident is not None:
        evenement = get_object_or_404(Evenement, nom_interne=ident)
    else:
        evenement = None

    if request.method == "POST":
        form = EvenementForm(request.POST, request.FILES, instance=evenement)
        if evenement:
            form.fields["affiche"].required = False

        if form.is_valid():
            evenement = form.save(commit=False)
            evenement.est_cloture = False  # Histoire de permettre un redémarrage en cas d’arrêt intempestif…
            evenement.save()
            if "affiche" in request.FILES:
                evenement.enreg_image(request.FILES["affiche"])

            return redirect("gestion:live:liste")
    else:
        form = EvenementForm(instance=evenement)
        if evenement:
            form.fields["affiche"].required = False

    c = {
        "form": form,
    }

    return render(request, "live/edition.html", c)


@permission_needed("gerer_live")
def suppression(request, ident):
    if request.ecole != Ecole.n7:
        raise Http404

    evenement = get_object_or_404(Evenement, nom_interne=ident)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            evenement.delete()
        return redirect("gestion:live:liste")

    message = """Êtes-vous sûr de vouloir supprimer cet événement ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer un événément")


@permission_needed("gerer_live")
def console(request):
    if request.ecole != Ecole.n7:
        raise Http404

    return render(request, "live/console.html", None)


@permission_needed("gerer_live")
def console_action(request):
    cur_time = int(time.time() * 1000)
    command = request.GET.get("cmd", "")

    live = Evenement.objects.filter(est_cloture=False).order_by("date_effective")[0:1]
    if live:
        live = live[0]
        live_time = time.mktime(live.date_effective.timetuple()) * 1000
    else:
        live = None
        live_time = 0

    if command == "end_live":
        if live:
            live.est_cloture = True
            live.save()

        return HttpResponse("", content_type="text/plain")

    if command == "postpone_live":
        try:
            delay = int(request.GET.get("delay", 0))
        except ValueError:
            delay = 0

        if delay:
            live.date_effective += timedelta(minutes=delay)
            live.save()

        return HttpResponse("", content_type="text/plain")

    if command == "remove_disco":
        ident = request.GET.get("stream", "invalid")
        connections = cache.get(f"live_stream_{ident}_connexions", [])

        if connections:
            connections = [conn for conn in connections if conn[2]]
            cache.set(f"live_stream_{ident}_connexions", connections, 3600)

        return HttpResponse("", content_type="text/plain")

    stream_statuses = []
    for stream in STREAMS:
        ident = stream["ident"]

        stream_statuses.append(
            {
                "name": ident,
                "ok": cache.get(f"live_stream_{ident}_actif", False),
                "connections": cache.get(f"live_stream_{ident}_connexions", []),
            }
        )

    return HttpResponse(
        json.dumps(
            {
                "cur_time": cur_time,
                "live_time": live_time,
                "stream_statuses": stream_statuses,
            },
            indent=None,
        ),
        content_type="text/plain",
    )


@permission_needed("gerer_live")
def test_live(request):
    return page_live(request, "prive" in request.GET)


def notification(request):
    if request.ecole != Ecole.n7 or request.method != "POST":
        raise Http404

    if request.META["REMOTE_ADDR"] not in settings.LIVE_AUTH_ALLOWED_IPS:
        return HttpResponseForbidden("Forbidden!", content_type="text/plain")

    if "mdp" in request.POST:
        live = Evenement.objects.filter(est_cloture=False).order_by("date_effective")[0:1]

        if live and live[0].prive:
            return HttpResponse(mdp_stream(), content_type="text/plain")

        return HttpResponse("", content_type="text/plain")

    for stream in STREAMS:
        ident = stream["ident"]
        statut = request.POST.get(ident, "")
        statut = statut.split(",")
        if not statut:
            continue

        actif = False
        connexions = cache.get(f"live_stream_{ident}_connexions", [])
        connexions_nouv = []

        if statut[0] == "1":
            actif = True
            connexions_nouv = []
            ips_actives = set(ip.strip() for ip in statut[1:])
            for hote, ip, _ in connexions:
                if ip in ips_actives:
                    ips_actives.discard(ip)
                    connexions_nouv.append((hote, ip, True))
                else:
                    connexions_nouv.append((hote, ip, False))

            for ip in ips_actives:
                try:
                    hote = gethostbyaddr(ip)[0]
                except Exception:
                    hote = ip

                connexions_nouv.append((hote, ip, True))

            connexions_nouv.sort()
        else:
            actif = False
            for hote, ip, _ in connexions:
                connexions_nouv.append((hote, ip, False))

        cache.set(f"live_stream_{ident}_actif", actif, 60)
        cache.set(f"live_stream_{ident}_connexions", connexions_nouv, 3600)

    return HttpResponse("ok", content_type="text/plain")
